<?php
// Buttons
$lang['save_btn'] = 'Save';
$lang['cancel_btn'] = 'Cancel';
$lang['delete_btn'] = 'Delete';
$lang['change_btn'] = 'Edit';
$lang['add_to_panel_btn'] = 'Add to panel';

// Pages
$lang['page_admin'] = 'Admin panel';

// Message
$lang['msg_head_error'] = 'Error';
$lang['msg_access_denied'] = 'You do not have permission! Do next:';
$lang['msg_back'] = 'Back to the previous page';
$lang['msg_relogin'] = 'Authorize again';
$lang['no data found'] = 'No event found';
$lang['Warning! Project is in development mode'] = '<b>Warning!</b> Project is in "development" mode';

// Menu
$lang['menu_back_text'] = '<span>«</span>Back';
$lang['menu_next_text'] = 'Next';
$lang['menu_title'] = 'Menu';
$lang['home_btn'] = 'Home';
$lang['site_btn'] = 'Go to website';
$lang['exit_btn'] = 'Exit';

// menu blocks
$lang['block_up_text'] = 'Minimize';
$lang['block_down_text'] = 'Maximize';

// search menu
$lang['search_text'] = 'Search ...';
$lang['search_title'] = 'Searching results:';

$lang['active'] = 'Yes';
$lang['no_active'] = 'No';

$lang['Translate manager'] = 'Translate manager';
$lang['Translations was saved'] = 'Translations was saved';
$lang['For example'] = 'For example';
$lang['Set default text'] = 'Set default text';
$lang['File imported. Records count %s. Please, check and save changes.'] = 'File imported. Records count %s. Please, check and save changes.';
$lang['Keys'] = 'Keys';
$lang['Translation key'] = 'Translation key';
$lang['Example'] = 'Example';
