<?php 

$lang['Delete users'] = 'Deleted user(s) "%s"';
$lang['Change pass'] = 'Change user password "%s"';
$lang['Change user data'] = 'Change profile "%s"';
$lang['Changed user rules'] = 'Change rules for user "%s"';