<style>
    #gdpr_popup {
        z-index: 999;
        color: black;
        background: white;
        position: fixed;
        bottom: 0;
        width: 100%;
        text-align: center;
        padding: 20px;
    }
</style>
<div id="gdpr_popup">
    <?= $this->lang->line('By using this web site you accept our use of cookies.') ?>&nbsp;&nbsp;
    <a href="#" onclick="document.getElementById('gdpr_popup').style.display='none';document.cookie='gdpr_accept=1';return false"><?= $this->lang->line('Accept') ?></a>
</div>